// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;
varying POS3 U;
varying highp vec3 position;
varying highp vec3 normal;
#ifdef FOG
varying vec4 fogColor;
#endif

#include "CLCS/uniformWorldConstants.h"
#include "CLCS/uniformPerFrameConstants.h"
#include "CLCS/uniformShaderConstants.h"
#include "CLCS/uniformRenderChunkConstants.h"
#include "CLCS/util.h"


LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

vec3 dayworldcolor=vec3(1.498,1.425,1.51);
vec3 duskworldcolor=vec3(1.9,1.2,0.5);
vec3 nightworldcolor=vec3(0.3,0.35,0.45);
vec3 rainworldcolor=vec3(0.41,0.52,0.63);
vec3 shadowcolor=vec3(0.35,0.38,0.51)*1.71;

bool UnWater(vec4 X){if((X.b>.15&&(X.b>X.g*1.2)&&(X.b>=X.r*1.8)&&(X.g>X.r*1.6))||(X.g*1.2>=X.b)&&(X.r*1.3<X.g)){return true;}else{return false;}}

void TimeSet(float rain,float uy,inout float daytime,inout float dusktime,inout float nighttime,inout float raintime){
float ussr=texture2D(TEXTURE_1,vec2(0.0,1.0)).r;
raintime= (1.0-pow(rain,6.0))*uy;
daytime=max((ussr-0.6)*2.5,0.0)*uy;
dusktime=(1.0-min((ussr-0.5)*2.5,1.0))*max((ussr-0.4)*2.5,0.0)*(1.0-raintime)*2.5*uy;
nighttime=(1.0-daytime)*(0.8-dusktime)*uy;}

highp float rand(highp vec2 X){ 
	return fract(cos(X.x + X.y *510.0) * 630.0);}

highp float noise(highp vec2 X){ 
   highp vec2 A = floor(X)*1.0; 
   highp vec2 B = fract(X)*1.0; 
   highp vec2 C = pow(B,vec2(3.0))*(3.0- 2.0*B); 
   highp float D = rand(A+vec2(0.0,0.0));
   highp float E = rand(A+vec2(1.0,0.0));
   highp float F = rand(A+vec2(0.0,1.0));
   highp float G = rand(A+vec2(1.0,1.0));
   highp float X1 = mix( mix(D, E, C.x),mix(F, G, C.x),C.y);
	return X1;}

float WP(highp vec3 v1,highp vec3 v2){return (v1.x*v2.x+v1.y*v2.y+v1.z*v2.z)/length(v1)/length(v2);}
highp float WV(highp vec3 A){
A.z+=TIME*.35;
highp vec2 B = vec2(A.z*2.0+A.y*1.5,A.x+A.y+A.z);
return sin(noise(B));}
vec3 WM(highp vec3 B){
highp float H=WV(B+vec3(.4,.3,.2))-WV(B);
return vec3(-H,1.,0.);}
highp float CS(highp vec2 X){
highp float A=1.;
highp float X1=.1;
X*=.11;
X+=vec2(1.8,3.)*(TIME*.027);
for(int B=0;B<3;B++){
X1+=noise(X)/A;
X*=7.1;
A*=9.4;
X-=TIME*.022*pow(A,.5)+noise(X*.3);}
return pow(abs(X1),3.);}
 
vec3 ToneMap(vec3 x) {
float A = 1.52;
float B = 0.1;
float C = 0.15 * 1.70;
float D = 0.4 * 1.30;
float E = 0.01 * 1.47;
float F = 0.4;

return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F));
}

vec3 WorldLight(vec3 color,float uy,float as,float bs,float cs,float ds,float ts){
   color=mix(color,color*dayworldcolor,as);
   color=mix(color,vec3(dot(color,vec3(0.55)))*nightworldcolor,bs*ts);
   color=mix(color,color*duskworldcolor,cs);
   color=mix(color,color*vec3(0.41,0.52,0.63),ds*uy*ts);
   color=ToneMap(color);
 return color;}

vec3 Shadow(vec3 color,float ds,float cw,float uy,float ux,float ts){
float shadowy =1.0-min(max((uy-0.89)*100.0,0.0),1.0);
float shadowx =1.0-min(max((ux-0.89)*20.0,0.0),1.0);
if(cw==0.0){shadowy=0.0;color*=1.2;}

color=mix(color,color*shadowcolor,shadowy*ts*(1.0-ds));
color=mix(color,color*shadowcolor,shadowx*ts*(1.0-shadowy)*uy*(1.0-ds));

 return color;
 }

 vec3 NomalTorch(vec3 color,vec3 cli,vec2 u, float ns){
    u.x=u.x*1.15+abs(0.05*sin(TIME))*0.8;
    color = mix(mix(color,cli,pow(u.x,3.5)),color,u.y);
    color=mix(color,cli,pow(u.x,3.5)*pow(ns,3.0));
 return color;}
 
 vec3 UnWaterLight(vec4 XFOG,vec3 color,inout vec3 cli){
    vec3 unwatercolor=vec3(0.3,0.4,0.95);
 if(UnWater(XFOG)){
cli=color*vec3(0.5,1.4,2.3)*2.6;
color*=unwatercolor;
highp vec3 svb=WM(position);
highp float bvb=abs(acos(WP(U,svb)));
color.rgb+=bvb*.1;}
return color;}
void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
	vec4 uvs =texture2D( TEXTURE_1, vec2(uv1.x*0.65,uv1.y));
#endif

float as,cs,bs,ds;
TimeSet(FOG_CONTROL.y,uv1.y,as,cs,bs,ds);
float ts=1.0-uv1.x;
 
float ux=color.g+0.35;
	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
	
vec4 inColor = color;
vec4 f=FOG_COLOR;
if((f.r>f.g)&&(f.b>f.g)&&(f.b>f.r)&&(f.r<.05&&f.b<.05&&f.g<.05)){diffuse.rgb*=vec3(1.4,0.6,1.);}
if((f.b==f.g)&&(f.b*1.5<f.r)){diffuse.rgb*=vec3(1.7,0.9,0.7);}

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse *= texture2D( TEXTURE_1, uv1 );
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif
	
	diffuse.rgb *= inColor.rgb;
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

#ifdef BLEND
#ifdef FANCY
if(color.r<color.g&&color.r*1.1<color.b&&color.g<color.b*1.25&&color.b*color.b>color.r*color.g){
vec3 svb=WM(position);
float bvb=abs(acos(WP(U,svb)));
vec4 cd=vec4(1);
vec4 cn=vec4(0,.5,.3,1);
vec4 cq=vec4(.7,.2,.1,1);
vec4 cr=vec4(.5,.5,.5,1);
vec4 cc=mix(mix(mix(cd,cq,as),cn,bs),cr,cs);
highp float J=CS(U.xz*1.7*bvb);
cc=mix(cc,cc*.7,J)*1.3;
cc.a=max(.6-J,0.);
diffuse=mix(mix(color*vec4(0.23,1.18,0.546,0.05),color*0.2*vec4(0.05,0.2,0.5,0.2),bs),cc,cc.a)*(1.0-bs);
diffuse+=bvb*.1;}
ux=color.r+color.g+color.b;
#endif
#endif

if(color.r==color.g){diffuse.rgb*=0.9;}

vec3 cli=diffuse.rgb*vec3(1.2,0.7,0.1)*1.5;
diffuse.rgb=UnWaterLight(FOG_COLOR,diffuse.rgb,cli);
#ifndef BLEND
diffuse.rgb=Shadow(diffuse.rgb,ds,color.w,uv1.y,ux,ts);
#endif
diffuse.rgb=NomalTorch(diffuse.rgb,cli,uv1,(1.0-as)*(1.0-cs));
diffuse.rgb=WorldLight(diffuse.rgb,uv1.y,as,bs,cs,ds,ts);

#ifdef FOG
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a*0.5);
#endif

	gl_FragColor = diffuse;
	
#endif // BYPASS_PIXEL_SHADER
}
