// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "vertexVersionCentroid.h"
#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		_centroid out vec2 uv0;
		_centroid out vec2 uv1;
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

#ifndef BYPASS_PIXEL_SHADER
	varying vec4 color;
#endif
varying vec4 fC;
varying POS3 U;
varying highp vec3 position;
varying highp vec3 normal;
#ifdef FOG
	varying vec4 fogColor;
#endif

#include "CLCS/uniformWorldConstants.h"
#include "CLCS/uniformPerFrameConstants.h"
#include "CLCS/uniformShaderConstants.h"
#include "CLCS/uniformRenderChunkConstants.h"
#include "CLCS/uniformEntityConstants.h"

attribute POS4 POSITION;
attribute vec4 COLOR;
attribute vec2 TEXCOORD_0;
attribute vec2 TEXCOORD_1;

const float rA = 1.0;
const float rB = 1.0;
const vec3 UNIT_Y = vec3(0,1,0);
const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange

void main()
{
    POS4 worldPos;
#ifdef AS_ENTITY_RENDERER
		POS4 pos = WORLDVIEWPROJ * POSITION;
		worldPos = pos;
#else
    worldPos.xyz = (POSITION.xyz * CHUNK_ORIGIN_AND_SCALE.w) + CHUNK_ORIGIN_AND_SCALE.xyz;
    worldPos.w = 1.0;
    POS4 pos = WORLDVIEW * worldPos;
    pos = PROJ * pos;
#endif
    gl_Position = pos;
     normal=worldPos.xyz;
    normal.y+=.0015;
    U=worldPos.xyz;
    position=POSITION.xyz;
#ifndef BYPASS_PIXEL_SHADER
    uv0 = TEXCOORD_0;
    uv1 = TEXCOORD_1;
	color = COLOR;
#endif


#ifndef BYPASS_PIXEL_SHADER
#ifdef ALPHA_TEST
POS3 ee = position.xyz + VIEW_POS;
if(color.g + color.g > color.r + color.b){
gl_Position.x += sin(TIME * 6.0 + ee.x+ee.z+ee.x+ee.z+ee.y) * sin(ee.z) * 0.02;
if(FOG_CONTROL.x < 0.55 && FOG_CONTROL.x > 0.1){
gl_Position.x += sin(TIME * 6.0 + ee.x+ee.z+ee.x+ee.z+ee.y) * sin(ee.x) * 0.03;
}}
#endif
#endif


///// find distance from the camera

#if defined(FOG) || defined(BLEND)
	#ifdef FANCY
		vec3 relPos = -worldPos.xyz;
		float cameraDepth = length(relPos);
	#else
		float cameraDepth = pos.z;
	#endif
#endif

///// apply fog

#ifdef FOG
	float len = cameraDepth / RENDER_DISTANCE;
	#ifdef ALLOW_FADE
		len += RENDER_CHUNK_FOG_ALPHA;
	#endif

    fogColor.rgb = FOG_COLOR.rgb;
	fogColor.a = clamp((len - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 0.0, 1.0);
#endif
if((FOG_COLOR.b>.15&&(FOG_COLOR.b>FOG_COLOR.g*1.2)&&(FOG_COLOR.b>=FOG_COLOR.r*1.8)&&(FOG_COLOR.g>FOG_COLOR.r*1.6))||(FOG_COLOR.g*1.2>=FOG_COLOR.b)&&(FOG_COLOR.r*1.3<FOG_COLOR.g)){gl_Position.xy+=.5*sin(TIME*.7+U.z+U.z+U.x+cos(TIME*.48+U.z))*sin(TIME*.92+U.x)*.17;}
#ifdef BLEND
	bool shouldBecomeOpaqueInTheDistance = color.a < 0.95;
	if(shouldBecomeOpaqueInTheDistance) {
		#ifdef FANCY
			float cameraDist = cameraDepth / FAR_CHUNKS_DISTANCE;
			color = COLOR;
		#else
			vec4 surfColor = vec4(color.rgb, 1.0);
			color = surfColor;
				
			vec3 relPos = -worldPos.xyz;
			float camDist = length(relPos);
			float cameraDist = camDist / FAR_CHUNKS_DISTANCE;
		#endif //FANCY
		
		float alphaFadeOut = clamp(cameraDist, 0.0, 1.0);
		color.a = mix(color.a, 1.0, alphaFadeOut);
	}
#endif

#ifndef BYPASS_PIXEL_SHADER
	#ifndef FOG
		// If the FOG_COLOR isn't used, the reflection on NVN fails to compute the correct size of the constant buffer as the uniform will also be gone from the reflection data
		color.rgb += FOG_COLOR.rgb * 0.000001;
	#endif
#endif
}
